import importlib
import json
import os
import sys
import traceback

try:
    import sgtk
    usingShotgun = True
except:
    usingShotgun = False

# The dictionary containing all the NIM 3rd party plugins
_NIM_3RD_PARTY_PLUGINS = {
    'Maya': '.nim_maya',
    'Nuke': '.nim_nuke',
    'C4D': '.nim_c4d',
    '3dsMax': '.nim_3dsmax',
    'Houdini': '.nim_houdini'
}


def _get_nim_module_as_string():
    """
    Getting the correct namespace for NIM.

    :return: The namespace for nim as a string
    """
    nim_mode_base = 'nim_core'

    try:
        nim_module_string = nim_mode_base + '.py' + sys.version_info.major

        importlib.import_module(nim_module_string)

        return nim_module_string
    except:
        return nim_mode_base


try:
    # This will be work in the older and new version of nim.
    nim = importlib.import_module('.nim', _get_nim_module_as_string())

    using_nim = True
except:
    using_nim = False


def get_nim_info_from_scene(nim_info):
    if nim_info.app() == "C4D":
        nim_c4d_plugin_id = 1032427
        args = (nim_info, nim_c4d_plugin_id)
    else:
        args = (nim_info,)

    nim_plugin = importlib.import_module(
        _NIM_3RD_PARTY_PLUGINS[nim_info.app()],
        _get_nim_module_as_string()
    )

    nim_plugin.get_vars( *args )

def getInfoFromShotgun( context ):
    shotgunInfo = {}

    if context.user is not None:
        # login is not guaranteed to be in the user dict
        shotgunInfo[ "UserLogin" ] = context.user.get( "login", "" )
        shotgunInfo[ "UserName" ] = context.user[ "name" ]

    if context.project is not None:
        shotgunInfo[ "ProjectName" ] = context.project[ "name" ]
        shotgunInfo[ "ProjectId" ] = context.project[ "id" ]

    if context.entity is not None:
        shotgunInfo[ "EntityName" ] = context.entity[ "name" ]
        shotgunInfo[ "EntityId" ] = context.entity[ "id" ]
        shotgunInfo[ "EntityType" ] = context.entity[ "type" ]

    if context.task is not None:
        shotgunInfo[ "TaskName" ] = context.task[ "name" ]
        shotgunInfo[ "TaskId" ] = context.task[ "id" ]

    return shotgunInfo

def writeInfoFile( tempPath, pipelineToolsInfo ):
    with open( os.path.join( tempPath, "pipeline_tools_info.json" ), "w" ) as fileHandle:
        fileHandle.write( json.dumps( pipelineToolsInfo ) )

def getInfo( tempPath ):
    pipelineToolsInfo = {
        "NIM" : {},
        "Shotgun" : {}
    }

    if using_nim:
        nim_info = nim.NIM()
        get_nim_info_from_scene(nim_info)
        # If there's actually info in this scene tied to NIM, pass it along, otherwise ensure NIM stays an empty dict
        if nim_info.nim.get("job", {}).get("ID", ""):
            pipelineToolsInfo["NIM"] = nim_info.nim

    if usingShotgun:
        context = sgtk.platform.current_engine().context
        pipelineToolsInfo[ "Shotgun" ] = getInfoFromShotgun( context )

    writeInfoFile( tempPath, pipelineToolsInfo )
